from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Components.ServiceScan import ServiceScan as CScan
from Components.ProgressBar import ProgressBar
from Components.Label import Label
from Components.NimManager import nimmanager, getConfigSatlist
from Components.ActionMap import ActionMap
from Components.FIFOList import FIFOList
from Components.Sources.FrontendInfo import FrontendInfo
from Plugins.Extensions.TheWebsiteParser.positioningsetup import PositionerSetup, TunerScreen, Diseqc, RotorNimSelection
from enigma import eServiceReference, eServiceCenter, eTimer, eDVBFrontendParametersSatellite, eComponentScan, \
eDVBSatelliteEquipmentControl, eDVBFrontendParametersTerrestrial, \
eDVBFrontendParametersCable, eConsoleAppContainer, eDVBResourceManager
from ServiceReference import ServiceReference

class ServiceScanSummary(Screen):
	skin = """
	<screen position="0,0" size="132,64">
		<widget name="Title" position="6,4" size="120,42" font="Regular;16" transparent="1" />
		<widget name="scan_progress" position="6,50" zPosition="1" borderWidth="1" size="56,12" backgroundColor="dark" />
		<widget name="Service" position="6,22" size="120,26" font="Regular;12" transparent="1" />
	</screen>"""
	
	def __init__(self, session, parent, showStepSlider = True):
                Screen.__init__(self, session, parent)
		
		self["Title"] = Label(parent.title or "ServiceScan")
		self["Service"] = Label("No Service")
		self["scan_progress"] = ProgressBar()
		
	def updateProgress(self, value):
		self["scan_progress"].setValue(value)

	def updateService(self, name):
		self["Service"].setText(name)

class ServiceScan(Screen):
	def ok(self):
		print "ok"
		if self["scan"].isDone():
		        print str(self["scan"])
		        if "None" in str(self["servicelist"].getCurrent()):
		               nimList = nimmanager.getNimListOfType("DVB-S")
		               if len(nimList) == 0:
			                self.session.open(MessageBox, _("No positioner capable frontend found."), MessageBox.TYPE_ERROR)
		               else:
			                if self.session.nav.RecordTimer.isRecording():
				                self.session.open(MessageBox, _("A recording is currently running. Please stop the recording before trying to configure the positioner."), MessageBox.TYPE_ERROR)
                                        else:
				                usableNims = []
				                for x in nimList:
				                	configured_rotor_sats = nimmanager.getRotorSatListForNim(x)
					                if len(configured_rotor_sats) != 0:
						        	usableNims.append(x)
				                if len(usableNims) == 1:
				                	#print str(usableNims) + " - hello"
                                                        self.session.open(PositionerSetup,usableNims[0],self.transponderdetailslist)			                
                                                        self.session.open(MessageBox, _("No channels were found when scanning in at the stored satellite position, the positioner setup will now be loaded for you to tune in the dish. Once the positioner setup has loaded, press red to tune the dish into the feeds transponder details before moving the dish manually. (transponder details are inputted automatically)(diseqc motor is required for this).  To exit the positioner setup simply press the exit button."),MessageBox.TYPE_WARNING)			                
                                                        #self.showtuner(usableNims[0])
				                elif len(usableNims) > 1:
                                                	self.session.open(RotorNimSelection)
				                else:
					        	self.session.open(MessageBox, _("No tuner is configured for use with a diseqc positioner!"), MessageBox.TYPE_ERROR)
		        if not "None" in str(self["servicelist"].getCurrent()):		                                       
                                self.ZapToChannel()
                                #self.session.openWithCallback(self.ZapToChannel, MessageBox, "Channel(s) were found during the scan for the given transponder, would you like to zap to it now?", MessageBox.TYPE_YESNO)                                
                                #self.session.openWithCallback(self.ZapToChannel, MessageBox, str(self.transponderdetailslist), MessageBox.TYPE_YESNO)                                
                        #else:
                                #self.session.openWithCallback(self.showtuner, MessageBox, "No Channel(s) were found during the scan for the given transponder, would you like to position your dish now? (This requires a Diseqc motor installed).", MessageBox.TYPE_YESNO)
                                #self.session.openWithCallback(self.showtuner, MessageBox, str(self.transponderdetailslist), MessageBox.TYPE_YESNO)                                
			self.close()       
	
        def LaunchPositionerSetup(self, retval):
                #we use this to launch the positioner setup if the user clicked yes to display it
                if retval:		
                                self.session.open(PositionerSetup, self.newusableNims[0], self.transponderdetailslist)                                                        				                	
               		

	#this gets called when the green button has been pressed, it is used to zap to a channel
	def ZapToChannel(self):
		#first we run a try block here, this is just incase the following function does not work and we can then catch any exceptions raised
		try:
			#we then strip white spaces from sat
			sat = self.transponderdetailslist[0].strip()
			#we get values from currently selected item within the menulist and put them into a list called frequencylist
			#first we create the list or set the list to nothing if it has already been created
			self.frequencylist = []
			#we then add everything we require to it, so first is the satellite name in which we created earlier, e.g. 192 for astra1 19.2
			self.frequencylist.append(sat)
			#we then want just the digits of the first item of thefrequency i.e. element zero, e.g. instead of 12.501H thefrequency[0] it would be 12501, notice the replace of the "." with "" i.e. nothing and also remove any digits
			self.frequencylist.append(self.transponderdetailslist[1])
			#we then append element 1 of thefrequency, this element will hold the letter H or V for horizontal or vertical transponder
			self.frequencylist.append(self.transponderdetailslist[2])
			#finally we add the symbol rate to frequencylist e.g. 27500
			self.frequencylist.append(self.transponderdetailslist[3])
			#finally we add in the FEC e.g. 2/3
			self.frequencylist.append(self.transponderdetailslist[4])
			#now that we have all values we can proceed, so the data should now look like this as an example:
			#thefrequency = 12148 (notice just digits are displayed here
			#thesymbollist[0] = 27500 (notice only digits are displayed here as we do not take the " - 3/4" or whatever)
			#sat = 282
			#looking at the values above, we know that the frequency is 12148, the symbol rate is 27500 and the satellite is 282 i.e. astra2 28.2east, this is actually a sky uk transponder for sky sports
			#now that we have that information we can begin to search through lamedb file for the frequency and symbol rate
			#Since the frequency and symbol rate in the lamedb are more thorough we will only look for part of these values by searching the first 6 charatcers of it, more is explained below 		
			#set up variable for previous line in file
			previousline = ""
			#we set up transpondertofind which will hold the data for the transponder in which we're trying to find, we set it to a default text which will get changed as a transponder is found
			transpondertofind = "Transponder Numbers Go Here"
			#we now need to srt up wsome lists to hold any transponder and channel data found, these are called transponderfoundlist and channelfoundlist
			transponderfoundlist = []
			channelfoundlist = []
			#we also create idline to hold a complete id for a channel
			idline = ""
			#011a0000:07e7:0002
			#	s 12148500:27500000:0:2:282:2:0			
			#we need read each line from the lamedb file, this file contains all of the information about transponders, satellite positions, channel names, channel data and much more information
			#notice here we do not need to set up a file pointer in the way we have done previously, there's no need to use urllib for this as we have a standard read from file function, the "r" means that we are reading data from the file
			file = open("/etc/enigma2/lamedb")
			#next we say for each line in the file we are going to do something
			for line in file:
				#remember earlier we extracted frequency, symbol rate and satellite info from the feed we wish to watch?  This is where it comes into use, we start with an if statement
				#here we say if the contents of thefrequency and thesymbollist[0] and sat are all in the current line in which we are checking within the file, we will then do something, otherwise we skip it
				#to note is that we check for a colon before the symbollist element zero and before sat, we do this because the symbollist and sat in which we have starts after the colon.  It stops us picking up any other value that may equal to the start of the symbolrate or sat within a frequency as they don't start with colon (:) in lamedb
				if self.transponderdetailslist[1] in line and ":"+self.transponderdetailslist[2] in line and ":"+sat in line:
					self["title"].setText(str(line))					
					#we tell transpondertofind to equal the previous line since this holds the transponder information
					transpondertofind = previousline.strip()
					#we tell previous line to equal the current line to avoid doing the next if statement when the for loop goes around again
					previousline = line				
					#continue

				#we then check if transpondertofind is in the previous line in which we are searching, if it is then we will add it to transponderfoundlist and also add the channel name from the current line for it to channelfoundlist
				if transpondertofind in previousline:		
					#the next lines of code involves some converting a value from denary to hexadecimal, this is because the values to make up the channel id  must be in hex format and not denary(i.e. none integer)
					#first we create a temporary list to hold the data we need for the channel ID, it is this data that will allow us to zap to a channel.  We get this data from what previousline variable is equal to at the time
					previousline = previousline.strip()
					channelIDdata = previousline.split(":")
					#idline = str(channelIDdata) + " ~ "
					#self["title"].setText(str(channelIDdata))
					#convert channelIDdata 4 and 5 from string to integer
					channelIDdata[5] = int(channelIDdata[5])
					channelIDdata[5] = hex(channelIDdata[5])
					channelIDdata[4] = int(channelIDdata[4])
					channelIDdata[4] = hex(channelIDdata[4])
					
					#next we tell previousline to equal the following, this will be the order channel id is made up in order to zap to a channel
					idline = "1:"+channelIDdata[5][2:]+":"+channelIDdata[4][2:]+":"+channelIDdata[0]+":"+channelIDdata[2]+":"+channelIDdata[3]+":"+channelIDdata[1]+":0:0:0"
					#ok so lets state what we have done above
					#first we tell previous line to equal 1 followed by a colon, all channel id start with one
					#next we add to it the value of fifth element of channelIDdata i.e. channelIDdata[5] converted into hexedecimal.  This value is usually always zero
					#next we add to it the value of fourth element of channelIDdata i.e. channelIDdata[4] converted into hexedecimal.  We do this because this data must be in hexadecimal form, it tells us what type of service it is in which we are wanting to zap to e.g. tv, radio or HD channel. NOTE we take the first two character off, this is because hex converts things with "0x" infront, e.g. 1 becomes 0x1 etc.
					#we then add on the channel number , this number is just an integer and no converting is needed, this number is element zero in channelIDdata list
					#we then add onto it the transponder number for this channel...at least i think this is the transponder number but i could be wrong
					#we then add on the number, again i am unsure what his number actually is but i think it is the SID(satellite ID), e.g. 002 is astra2 28.2east, 001 is astra1 19.2east and 13e is hotbird 13east etc.  As this is an id we do not convert it into hex
					#now we add on another number which again seems to be part of the transponder ID, i am not sure exactly what this number is but a guess would say it's too part of transponder ID, so channelIDdata[1] is part of transponder id (at a guess)
					#now the last values in which we need to give the box in order to change channel is a few characters at the end, these are three zeros split up by colon e.g. ":0:0:0"...again i am not too sure what these numbers mean, maybe they are for VID, PID etc. but i cannot be certain
					#OK WE NOW HAVE THE DATA TO SEND, we append previous line make up of the data to send to our transponderfoundlist
					transponderfoundlist.append(idline)
					#however we can just add the channel as it is to the channelfoundlist since that is just displayed as a string
					channelfoundlist.append(line)
					#notice some lines start with "p:", this is just provider information that we do not need for what we're doing, providers are e.g. bskyb or skyitalia
					if self.channelname in line:
						#we open up a messagebox, if the user clicks ok on the message box we then call finishedServiceSelection with the id line, we know this is the right line to use as it was found and channel name equals the current line
						self.session.openWithCallback(self.finishedServiceSelection(idline), MessageBox, "Channel found, if you would like to zap to channel "+self.channelname+" on satellite "+str(self.satellite[self["mylist"].getSelectedIndex()])+" please press ok on yes, otherwise select no.", MessageBox.TYPE_YESNO)
						#if the channel was found we close the file and then break out of the loop which stops us doing anymore further searching within the for loop, we also clear the lists incase user does another search
						del transponderfoundlist[:]
						del channelfoundlist[:]
						break				
				#we tell previous line to equal the line, this only changes  once the code above has gone through so effectively we can get the previous line from the lamedb file
				#the previous line will hold any information in which we need about channel id's
				previousline = line
			#here we close the file
			file.close()
			#we then do a check to see if the length of transponderfoundlist is greater than zero, if it is and the loop has ended we know that we have some channel data for a transponder.  therefore we call TransponderChannels screen
			if len(transponderfoundlist) > 0:
				#we give the screen two arguments, one being the list transpoderfoundlist and the other which is channelfoundlist holding any channels found on searched transponder
				self.session.open(TransponderChannels,transponderfoundlist,channelfoundlist)
			#if the length of the transponderfoundlist is equal to zero we know that no transponder was found and so we will display a message
			if len(transponderfoundlist) == 0:
				#here we call self.scansat to scan the satellite, we have a callback to only do the operation if theu ser clicked yes to launch the scan satellite screen
				self.session.openWithCallback(self.showtuner, MessageBox, "No channel or transponders found for the name "+self.channelname+" on satellite "+str(sat)+". Would you like to select a satellite and update your transponders/channels now?", MessageBox.TYPE_YESNO)			
			#we create a variable called FrequencyandSymbol, we then tell it to equal th string of thefrequency variable, we add on a colon and then we add in the fitst element from thesymbollist i.e. element zero
			#e.g. frequencyandsymbol may now say something like 10817000:22000000 <---that would find the transponder for the BBC1 NE & C service on astra2 28.2east, plus other BBC1 variants on that trnasponder
			#FrequencyandSymbol = str(thefrequency + ":" thesymbollist[0])

			#we show a messagebox, note that we show this messagebox and if user presses yes or no it will call update with a value, either true or false
			#self.session.openWithCallback(self.finishedServiceSelection, , MessageBox, "The channel "+self.channelname+" could not be found on satellite "+str(self.satellite[self["mylist"].getSelectedIndex()])+". Would you like to search this channels transponder?", MessageBox.TYPE_YESNO)
	
			#self.session.openWithCallback(self.finishedServiceSelection, SimpleChannelSelection, _("Select channel to refresh"))		
		except Exception, e:
			return
                        #self["title"].setText(str(e))
		#self["title"].setText(idline)
	
	def finishedServiceSelection(self, *args):
		#if ars were supplied i.e. an argument was given
		if args:
			#we tell the service to equal the first argument to string
			self.service = args[0].toString()
			#we then set the text of the title to argument zero and then we get the service reference i.e. channel name
			self["title"].setText(str(args[0].toString())+ " "+str(ServiceReference(args[0]).getServiceName().replace('\xc2\x86', '').replace('\xc2\x87', '')))	
			#we then call force refresh with self.session
			self.forceRefresh()
			
	def forceRefresh(self):
		#forcerefresh must be supplied with a session, if the self.session is none and if session is not equal to none we can tell self.session to equal session argument
		if self.service is not None:
			#so if the self.service variable equal to something we can now play the service, we do this by getting the name from self.service variable
			#notice the screen we use is self.session.nav.playServer, this allows us to play the service what self.servive equals to within eServiceReference on the tv, i.e. zap is complete
			self.session.nav.playService(eServiceReference(self.service))
			self.session.open(MessageBox,_("Channel zapped Successfully.  Your service should now be displayed in the background"), MessageBox.TYPE_INFO)
			#finally we return true if complete
			return True
		else:
			return False	
	
	def cancel(self):
		self.close()
		
	def __init__(self, session, scanList, transponderdetailslist):
		Screen.__init__(self, session)
		
		self.scanList = scanList
		self.transponderdetailslist = transponderdetailslist	
                self.service = ""
		self.session.nav.stopService()
		
		self["scan_progress"] = ProgressBar()
		self["scan_state"] = Label(_("scan state"))
		self["network"] = Label()
		self["transponder"] = Label()

		self["pass"] = Label("")
		self["servicelist"] = FIFOList(len=10)
		self["FrontendInfo"] = FrontendInfo()
 		
		self["actions"] = ActionMap(["OkCancelActions"], 
			{
				"ok": self.ok,
				"cancel": self.cancel
			})

		self.onFirstExecBegin.append(self.doServiceScan)

	def doServiceScan(self):
		self["scan"] = CScan(self["scan_progress"], self["scan_state"], self["servicelist"], self["pass"], self.scanList, self["network"], self["transponder"], self["FrontendInfo"], self.session.summary)

	def createSummary(self):
		print "ServiceScanCreateSummary"
		return ServiceScanSummary